---
title: Glossar
description: "Definiert Schlüsselbegriffe, die in den Texten der Arbeitsgruppe Plattformen verwendet werden."
---

Siehe auch: [Cloud Native Glossar](https://glossary.cncf.io/de/)

Bei Bezug auf die folgenden Definitionen außerhalb des Rahmens der Texte der Arbeitsgruppe muss beachtet werden, dass diese sowohl im Kontext der Cloud Native Computing Foundation (CNCF) als auch der Technical Advisory Group (TAG) App Delivery verfasst wurden. 

## Plattform
Eine Sammlung von Fähigkeiten, Dokumentationen und Tools, die die Entwicklung, das Ausliefern, den Betrieb und/oder die Verwaltung der Bereitstellung von Produkten und Diensten unterstützen. Eine Plattform kann Webportale, APIs, CLIs, Protokolldefinitionen, Dokumentationen, Standards und/oder Golden Path Templates umfassen. Wenn sie gut gemacht sind, ermöglichen Plattformen eine schnellere und zuverlässigere Bereitstellung der Anwendungen und Dienste eines Unternehmens.

Je nach Umfang und Zielgruppe einer Plattform wird sie manchmal auch als „Entwicklerplattform“, „interne Entwicklerplattform“, abgekürzt aus dem englischen "Internal Developer Platform" als "IDP", „Bereitstellungsplattform“, „App-Plattform“ oder sogar als „Cloud-Plattform“ bezeichnet. Der Begriff „Platform-as-a-Service (PaaS)“ wird auch häufig verwendet, um Plattformen zu beschreiben, die von außerhalb einer Organisation gekauft oder anderweitig übernommen werden und eine vom Anbieter betriebene, aber oft weniger anpassbare Plattformlösung bieten.

## Plattform-Engineering
Der Entwurf, Aufbau, Betrieb und die Weiterentwicklung einer Plattform. Eine Möglichkeit, diese Praxis zu betrachten, ist ein auf Empathie basierender Ansatz für die soziotechnische Gestaltung von Organisationen<sup><a href="https://hazelweakly.me/talks/qcon-sf-2023/slides#22">1</a></sup>. In diesem Sinne ist es ein kontinuierlicher Prozess, durch den eine Organisation lernt, wie und wo sie nicht nur extern, sondern auch intern investieren und strategische Geschäftsentscheidungen treffen kann.

## Plattform Team

Die Personen, die für den Aufbau und den Betrieb der Plattform(en) verantwortlich sind. Zu den Mitgliedern des Plattform Teams gehören **Plattformingenieure**, die sich auf die Entwicklung der Werkzeuge und Fähigkeiten konzentrieren, aus denen die Plattformerlebnisse bestehen. Es kann auch spezielle **Plattform-Produktmanager** umfassen, die sich auf die Erfüllung interner Kundenanforderungen konzentrieren und gleichzeitig die umfassenderen strategischen Ziele des Unternehmens unterstützen. Im Zuge der Weiterentwicklung der Plattform können die Plattform Teams um weitere spezialisierte Rollen wie Systemoperatoren, QA-Analysten, UI/UX-Designer, technische Redakteure und Entwicklerbotschafter erweitert werden.

## DevOps
„DevOps ist eine Methode, bei der Teams den gesamten Prozess von der Anwendungsentwicklung bis zum Produktionsbetrieb selbst in die Hand nehmen.<sup><a href="https://glossary.cncf.io/devops/">2</a></sup>" Während DevOps-Praktiken von Teams ohne die Entwicklung einer speziellen Plattform implementiert werden können, kann es sinnvoll sein, Plattform-Engineering als einen Ansatz zur Skalierung von DevOps-Prinzipien durch die Bereitstellung und den Betrieb einer einheitlichen Plattform zu betrachten, die dem gesamten Unternehmen dient. Diese gemeinsame Plattform zielt darauf ab, die Entwicklungs-, Bereitstellungs- und Betriebsprozesse zu rationalisieren und eine standardisierte und effiziente Umgebung für die Softwarebereitstellung zu schaffen. Obwohl DevOps und Plattform-Engineering in den Zielen der Optimierung der Softwarebereitstellung und der Betriebsleistung konvergieren, liegt der Schwerpunkt des Plattform-Engineerings eindeutig auf der Entwicklung eines greifbaren Produkts - der Plattform selbst - um diese Ziele zu erreichen.

## Plattform Nutzer
Die Personen, die die Fähigkeiten der Plattform direkt nutzen, einschließlich, aber nicht beschränkt auf App-Entwickler, App-Betreiber, Datenwissenschaftler, Betreiber kommerzieller Standardsoftware und Wissensarbeiter - alle, die Software auf der Plattform ausführen, von der Plattform bereitgestellte Fähigkeiten nutzen oder Einblicke in die Plattformnutzung benötigen. Zu den Plattformnutzern können auch andere Plattformingenieure gehören, die übergeordnete Plattformdienste auf der Grundlage von Fähigkeiten auf niedrigerer Ebene erstellen.

## Portal
Eine webbasierte Schnittstelle, die einen zentralen Zugang zu einer Vielzahl von Ressourcen, Tools und Diensten bietet. Sie kann als Ausgangspunkt für eine Vielzahl von Benutzern dienen, um die Fähigkeiten der zugrunde liegenden Plattform effizient zu verwalten und mit ihnen zu interagieren. Ein Portal dient dazu, das Benutzererlebnis durch eine benutzerfreundliche Oberfläche zu verbessern, die komplexe Prozesse vereinfacht und Selbstbedienungsfunktionen fördert. 

## Fähigkeiten der Plattform
Die spezifischen Ergebnisse für den Benutzer, oder **_was_** eine Plattform bietet. Diese sind nicht zu verwechseln mit Plattformeigenschaften, die beschreiben, **_wie_** die Fähigkeiten funktionieren. Diese Fähigkeiten können auf verschiedenen Abstraktionsebenen liegen (z. B. eine einzelne Datenbank oder eine Testumgebung mit einer Datenbank) und von verschiedenen Anbietern bereitgestellt werden. Mit zunehmender Reife der Plattformen werden diese Anbieter im Allgemeinen bestrebt sein, Fähigkeiten über Selbstbedienung anzubieten, angefangen bei der Auffindbarkeit der verfügbaren Fähigkeiten bis hin zum konsistenten Nutzererlebnis über alle Fähigkeiten hinweg. Die Fähigkeiten selbst sind oft recht langlebig, während sich die Anbieter und die Implementierung schneller weiterentwickeln können. Es ist zum Beispiel unwahrscheinlich, dass ein Unternehmen keine Testumgebungen mehr benötigt, aber es kann sich dahin entwickeln, dass es statt VM-basierter Lösungen containerisierte Lösungen anbietet.

## Anbieter von Plattformfähigkeiten
Eine Gruppe von Personen, die eine von der Plattform angebotene Fähigkeit entwickeln und pflegen. Bei den Anbietern kann es sich um externe Organisationen oder interne Teams handeln, und in kleineren Organisationen sind es oft dieselben Personen, die auch die breitere Plattform entwickeln. Wenn Plattformen reifen, profitieren sie von der Beibehaltung von Abstraktionen für die Anbieter, um ein „Lock-in“ zu verhindern und die Entwicklung hin zur „Thinnest Viable Platform“ voranzutreiben.

## Eigenschaften der Plattform
Bezieht sich darauf, **_wie_** die Plattform und ihre Fähigkeiten funktionieren und was in Bezug auf funktionsübergreifende Anforderungen, nicht-funktionale Anforderungen oder einfach Qualitäten erwartet werden kann. Beispiele hierfür sind die Zuverlässigkeit oder Leistung von betriebenen Diensten, die anhand von Service Level Objectives (SLOs) gemessen werden können, die Sicherheit, die anhand der Zeit bis zur Abschwächung identifizierter Risiken gemessen werden kann, oder die Beobachtbarkeit, die sowohl zur Fehlersuche als auch zur Berichterstattung über die Nutzung der Plattform verwendet werden kann. Eigenschaften werden oft mit Fähigkeiten verwechselt, da einige Konzepte wie die Beobachtbarkeit als Fähigkeit (z. B. OTel Operator, der zur Erfassung von Anwendungstelemetrie bereitgestellt wird) und als deklarierte Qualität (z. B. Plattformmetriken zur Messung und Meldung der Betriebszeit des bereitgestellten OTel Operators) angeboten werden können.

## Kognitive Belastung
Eine Quantifizierung der mentalen Kosten, die einem Nutzer entstehen, bevor er von einer Plattformfähigkeit profitieren kann. Unter dem Oberbegriff der kognitiven Belastung gibt es eigentlich drei Arten von Belastung: die eigentliche, die intrinsische und die fremde. Unternehmen sind am gesündesten, wenn Plattformen es den Nutzern ermöglichen, sich auf die eigentliche Belastung (aufgabenspezifische Problemlösung) zu konzentrieren, während die intrinsische Belastung (Einarbeitung in neue Informationen oder Prozesse zur Erledigung der Aufgabe) vereinfacht und die externe Belastung (Ablenkungen von der fokussierten Aufgabe, manchmal auch als “[Yak Shaving](https://en.wiktionary.org/wiki/yak_shaving#:~:text=yak%20shaving%20(uncountable),to%20solve%20a%20larger%20problem.)” bezeichnet) minimiert wird.

## Thinnest Viable Platform (TVP)
Ein Konzept, das ursprünglich in dem Buch _Team Topologies_ von Matthew Skelton und Manuel Pais definiert wurde und das Unternehmen dazu ermutigt, ein ausgewogenes Verhältnis zwischen einer kleinen, aber effektiven Plattform zu finden. Auf diese Weise können sie die Softwarebereitstellung für die auf der Plattform aufbauenden Teams beschleunigen und vereinfachen und gleichzeitig ihre umfassenderen Geschäftsziele erreichen. Das Modell ermutigt Plattformen, sich auf die einzigartigen Anforderungen des Unternehmens zu konzentrieren und routinemäßig Drittanbieter zu integrieren, die die Komplexität und die Betriebskosten der Plattform reduzieren können.
